@@ -34,15 +34,30 @@
 #include <errno.h>
 
 #include <glib/gi18n-lib.h>
+
+#ifdef NM_OPENVPN_OLD
+#define NM_VPN_LIBNM_COMPAT
 #include <nm-setting-connection.h>
 #include <nm-setting-8021x.h>
 #include <nm-utils.h>
 #include <nm-ui-utils.h>
 
+#define OPENVPN_EDITOR_PLUGIN_ERROR                     NM_SETTING_VPN_ERROR
+#define OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY    NM_SETTING_VPN_ERROR_INVALID_PROPERTY
+
+#else /* !NM_OPENVPN_OLD */
+
+#include <NetworkManager.h>
+#include <nma-ui-utils.h>
+
+#define OPENVPN_EDITOR_PLUGIN_ERROR                     NM_CONNECTION_ERROR
+#define OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY    NM_CONNECTION_ERROR_INVALID_PROPERTY
+#endif
+
 #include "auth-helpers.h"
 #include "nm-openvpn.h"
-#include "src/nm-openvpn-service.h"
-#include "common/utils.h"
+#include "../src/nm-openvpn-service-defines.h"
+#include "../common/utils.h"
 
 #define BLOCK_HANDLER_ID "block-handler-id"
 
@@ -55,7 +70,7 @@ show_password (GtkToggleButton *togglebutton, GtkEntry *password_entry)
 static GtkWidget *
 setup_secret_widget (GtkBuilder *builder,
                      const char *widget_name,
-                     NMSettingVPN *s_vpn,
+                     NMSettingVpn *s_vpn,
                      const char *secret_key)
 {
 	GtkWidget *widget;
@@ -148,7 +163,7 @@ tls_cert_changed_cb (GtkWidget *widget, gpointer data)
 static void
 tls_setup (GtkBuilder *builder,
            GtkSizeGroup *group,
-           NMSettingVPN *s_vpn,
+           NMSettingVpn *s_vpn,
            const char *prefix,
            GtkWidget *ca_chooser,
            ChangedCallback changed_cb,
@@ -233,8 +248,8 @@ tls_setup (GtkBuilder *builder,
 
 static void
 pw_setup (GtkBuilder *builder,
-          GtkSizeGroup *group, 
-          NMSettingVPN *s_vpn,
+          GtkSizeGroup *group,
+          NMSettingVpn *s_vpn,
           const char *prefix,
           ChangedCallback changed_cb,
           gpointer user_data)
@@ -269,7 +284,7 @@ pw_setup (GtkBuilder *builder,
 void
 tls_pw_init_auth_widget (GtkBuilder *builder,
                          GtkSizeGroup *group,
-                         NMSettingVPN *s_vpn,
+                         NMSettingVpn *s_vpn,
                          const char *contype,
                          const char *prefix,
                          ChangedCallback changed_cb,
@@ -326,7 +341,7 @@ tls_pw_init_auth_widget (GtkBuilder *builder,
 void
 sk_init_auth_widget (GtkBuilder *builder,
                      GtkSizeGroup *group,
-                     NMSettingVPN *s_vpn,
+                     NMSettingVpn *s_vpn,
                      ChangedCallback changed_cb,
                      gpointer user_data)
 {
@@ -391,9 +406,6 @@ sk_init_auth_widget (GtkBuilder *builder,
 	g_object_unref (store);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? 0 : active);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "sk_dir_help_label"));
-	gtk_size_group_add_widget (group, widget);
-
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "sk_local_address_entry"));
 	gtk_size_group_add_widget (group, widget);
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (changed_cb), user_data);
@@ -442,8 +454,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_PLUGIN_UI_ERROR,
-		             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             OPENVPN_EDITOR_PLUGIN_ERROR,
+		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_CA);
 		return FALSE;
 	}
@@ -453,8 +465,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_PLUGIN_UI_ERROR,
-		             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             OPENVPN_EDITOR_PLUGIN_ERROR,
+		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_CERT);
 		return FALSE;
 	}
@@ -465,8 +477,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_PLUGIN_UI_ERROR,
-		             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             OPENVPN_EDITOR_PLUGIN_ERROR,
+		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_KEY);
 		return FALSE;
 	}
@@ -487,8 +499,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	if (encrypted && secrets_required) {
 		if (!gtk_entry_get_text_length (GTK_ENTRY (widget))) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_CERTPASS);
 			return FALSE;
 		}
@@ -514,16 +526,16 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_USERNAME);
 			return FALSE;
 		}
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD)) {
 		if (!validate_file_chooser (builder, "pw_ca_cert_chooser")) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_CA);
 			return FALSE;
 		}
@@ -531,16 +543,16 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_USERNAME);
 			return FALSE;
 		}
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
 		if (!validate_file_chooser (builder, "sk_key_chooser")) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_STATIC_KEY);
 			return FALSE;
 		}
@@ -549,8 +561,8 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_LOCAL_IP);
 			return FALSE;
 		}
@@ -559,8 +571,8 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_REMOTE_IP);
 			return FALSE;
 		}
@@ -575,7 +587,7 @@ update_from_filechooser (GtkBuilder *builder,
                          const char *key,
                          const char *prefix,
                          const char *widget_name,
-                         NMSettingVPN *s_vpn)
+                         NMSettingVpn *s_vpn)
 {
 	GtkWidget *widget;
 	char *tmp, *filename;
@@ -597,7 +609,7 @@ update_from_filechooser (GtkBuilder *builder,
 }
 
 static void
-update_tls (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
+update_tls (GtkBuilder *builder, const char *prefix, NMSettingVpn *s_vpn)
 {
 	GtkWidget *widget;
 	NMSettingSecretFlags pw_flags;
@@ -623,7 +635,7 @@ update_tls (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
 }
 
 static void
-update_pw (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
+update_pw (GtkBuilder *builder, const char *prefix, NMSettingVpn *s_vpn)
 {
 	GtkWidget *widget;
 	NMSettingSecretFlags pw_flags;
@@ -660,7 +672,7 @@ update_pw (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
 gboolean
 auth_widget_update_connection (GtkBuilder *builder,
                                const char *contype,
-                               NMSettingVPN *s_vpn)
+                               NMSettingVpn *s_vpn)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -755,12 +767,13 @@ tls_default_filter (const GtkFileFilterInfo *filter_info, gpointer data)
 	if (!ext)
 		return FALSE;
 
-	if (pkcs_allowed && !strcmp (ext, ".p12") && is_pkcs12 (filter_info->filename)) {
+	if (pkcs_allowed && g_str_has_suffix (ext, ".p12") && is_pkcs12 (filter_info->filename)) {
 		g_free (ext);
 		return TRUE;
 	}
 
-	if (strcmp (ext, ".pem") && strcmp (ext, ".crt") && strcmp (ext, ".key") && strcmp (ext, ".cer")) {
+	if (!g_str_has_suffix (ext, ".pem") && !g_str_has_suffix (ext, ".crt") &&
+		!g_str_has_suffix (ext, ".key") && !g_str_has_suffix (ext, ".cer")) {
 		g_free (ext);
 		return FALSE;
 	}
@@ -844,7 +857,7 @@ sk_default_filter (const GtkFileFilterInfo *filter_info, gpointer data)
 	ext = g_ascii_strdown (p, -1);
 	if (!ext)
 		return FALSE;
-	if (strcmp (ext, ".key")) {
+	if (!g_str_has_suffix (ext, ".key")) {
 		g_free (ext);
 		return FALSE;
 	}
@@ -885,6 +898,7 @@ static const char *advanced_keys[] = {
 	NM_OPENVPN_KEY_PORT,
 	NM_OPENVPN_KEY_COMP_LZO,
 	NM_OPENVPN_KEY_MSSFIX,
+	NM_OPENVPN_KEY_FLOAT,
 	NM_OPENVPN_KEY_TUNNEL_MTU,
 	NM_OPENVPN_KEY_FRAGMENT_SIZE,
 	NM_OPENVPN_KEY_TAP_DEV,
@@ -905,6 +919,9 @@ static const char *advanced_keys[] = {
 	NM_OPENVPN_KEY_TLS_REMOTE,
 	NM_OPENVPN_KEY_REMOTE_RANDOM,
 	NM_OPENVPN_KEY_REMOTE_CERT_TLS,
+	NM_OPENVPN_KEY_PING,
+	NM_OPENVPN_KEY_PING_EXIT,
+	NM_OPENVPN_KEY_PING_RESTART,
 	NULL
 };
 
@@ -927,12 +944,12 @@ advanced_dialog_new_hash_from_connection (NMConnection *connection,
                                           GError **error)
 {
 	GHashTable *hash;
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	const char *secret, *flags;
 
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	s_vpn = nm_connection_get_setting_vpn (connection);
 	nm_setting_vpn_foreach_data_item (s_vpn, copy_values, hash);
 
 	/* HTTP Proxy password is special */
@@ -1203,8 +1220,6 @@ tls_auth_toggled_cb (GtkWidget *widget, gpointer user_data)
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_auth_checkbutton"));
 	use_auth = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_dir_help_label"));
-	gtk_widget_set_sensitive (widget, use_auth);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "direction_label"));
 	gtk_widget_set_sensitive (widget, use_auth);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_auth_label"));
@@ -1222,6 +1237,9 @@ tls_auth_toggled_cb (GtkWidget *widget, gpointer user_data)
 #define DEVICE_TYPE_IDX_TUN     0
 #define DEVICE_TYPE_IDX_TAP     1
 
+#define PING_EXIT    0
+#define PING_RESTART 1
+
 static void
 proxy_type_changed (GtkComboBox *combo, gpointer user_data)
 {
@@ -1361,6 +1379,19 @@ dev_checkbox_toggled_cb (GtkWidget *check, gpointer user_data)
 	device_name_changed_cb (GTK_ENTRY (entry), ok_button);
 }
 
+static void
+ping_exit_restart_checkbox_toggled_cb (GtkWidget *check, gpointer user_data)
+{
+	GtkBuilder *builder = (GtkBuilder *) user_data;
+	GtkWidget *combo, *spin;
+
+	combo = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_combo"));
+	spin = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_spinbutton"));
+
+	checkbox_toggled_update_widget_cb (check, combo);
+	checkbox_toggled_update_widget_cb (check, spin);
+}
+
 #define TA_DIR_COL_NAME 0
 #define TA_DIR_COL_NUM 1
 
@@ -1595,6 +1626,12 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 	}
 
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_FLOAT);
+	if (value && !strcmp (value, "yes")) {
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "float_checkbutton"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+	}
+
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PROTO_TCP);
 	if (value && !strcmp (value, "yes")) {
 		widget = GTK_WIDGET (gtk_builder_get_object (builder, "tcp_checkbutton"));
@@ -1742,6 +1779,74 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 		gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 2);
 	}
 
+	/* ping */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_checkbutton"));
+	spin = GTK_WIDGET (gtk_builder_get_object (builder, "ping_spinbutton"));
+	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (checkbox_toggled_update_widget_cb), spin);
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PING);
+	if (value && *value) {
+		long int tmp;
+
+		errno = 0;
+		tmp = strtol (value, NULL, 10);
+		if (errno == 0 && tmp > 0 && tmp < 65536) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_spinbutton"));
+			gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gdouble) tmp);
+			gtk_widget_set_sensitive (widget, TRUE);
+		}
+	} else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_spinbutton"));
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), 30.0);
+		gtk_widget_set_sensitive (widget, FALSE);
+	}
+
+	/* ping-exit / ping-restart */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_checkbutton"));
+	ping_exit_restart_checkbox_toggled_cb (widget, builder);
+	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (ping_exit_restart_checkbox_toggled_cb), builder);
+
+	combo = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_combo"));
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PING_EXIT);
+	if (value && *value)
+		active = PING_EXIT;
+	else {
+		value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PING_RESTART);
+		active = PING_RESTART;
+	}
+
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("ping-exit"), -1);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("ping-restart"), -1);
+	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
+	g_object_unref (store);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), active);
+
+	if (value && *value) {
+		long int tmp;
+
+		errno = 0;
+		tmp = strtol (value, NULL, 10);
+		if (errno == 0 && tmp > 0 && tmp < 65536) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_spinbutton"));
+			gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gdouble) tmp);
+			gtk_widget_set_sensitive (widget, TRUE);
+		}
+	} else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_spinbutton"));
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), 30.0);
+		gtk_widget_set_sensitive (widget, FALSE);
+	}
+
 out:
 	g_free (ui_file);
 	return dialog;
@@ -1868,6 +1973,10 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_MSSFIX), g_strdup ("yes"));
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "float_checkbutton"));
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_FLOAT), g_strdup ("yes"));
+
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tcp_checkbutton"));
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_PROTO_TCP), g_strdup ("yes"));
@@ -1973,12 +2082,41 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 				gtk_tree_model_get (model, &iter, TA_DIR_COL_NUM, &direction, -1);
 				if (direction >= 0) {
 					g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_TA_DIR),
-					                     g_strdup_printf ("%d", direction));					
+					                     g_strdup_printf ("%d", direction));
 				}
 			}
 		}
 	}
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_checkbutton"));
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+		int ping_val;
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_spinbutton"));
+		ping_val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget));
+
+		g_hash_table_insert (hash,
+		                     g_strdup (NM_OPENVPN_KEY_PING),
+		                     g_strdup_printf ("%d", ping_val));
+	}
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_checkbutton"));
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+		int ping_exit_type, ping_val;
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_combo"));
+		ping_exit_type = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_spinbutton"));
+		ping_val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget));
+
+		g_hash_table_insert (hash,
+		                     ping_exit_type == PING_EXIT ?
+		                       g_strdup (NM_OPENVPN_KEY_PING_EXIT) :
+		                       g_strdup (NM_OPENVPN_KEY_PING_RESTART),
+		                     g_strdup_printf ("%d", ping_val));
+	}
+
 	return hash;
 }
 
