@@ -27,6 +27,7 @@
 #include <sys/types.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <arpa/inet.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -35,13 +36,32 @@
 
 #include <glib/gi18n-lib.h>
 
+#ifdef NM_OPENVPN_OLD
+#define NM_VPN_LIBNM_COMPAT
 #include <nm-setting-vpn.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-ip4-config.h>
+#include <nm-utils.h>
+
+#define nm_simple_connection_new nm_connection_new
+#define NM_SETTING_IP_CONFIG NM_SETTING_IP4_CONFIG
+#define NM_SETTING_IP_CONFIG_METHOD NM_SETTING_IP4_CONFIG_METHOD
+#define NMSettingIPConfig NMSettingIP4Config
+
+#define OPENVPN_EDITOR_PLUGIN_ERROR                     NM_SETTING_VPN_ERROR
+#define OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN    NM_SETTING_VPN_ERROR_UNKNOWN
+
+#else /* !NM_OPENVPN_OLD */
+
+#include <NetworkManager.h>
+
+#define OPENVPN_EDITOR_PLUGIN_ERROR                     NM_CONNECTION_ERROR
+#define OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN    NM_CONNECTION_ERROR_FAILED
+#endif
 
 #include "import-export.h"
 #include "nm-openvpn.h"
-#include "../src/nm-openvpn-service.h"
+#include "../src/nm-openvpn-service-defines.h"
 #include "../common/utils.h"
 
 #define AUTH_TAG "auth "
@@ -56,6 +76,7 @@
 #define KEYSIZE_TAG "keysize "
 #define CLIENT_TAG "client"
 #define COMP_TAG "comp-lzo"
+#define FLOAT_TAG "float"
 #define DEV_TAG "dev "
 #define DEV_TYPE_TAG "dev-type "
 #define FRAGMENT_TAG "fragment "
@@ -63,7 +84,11 @@
 #define KEY_TAG "key "
 #define KEY_BLOB_START_TAG "<key>"
 #define KEY_BLOB_END_TAG "</key>"
+#define KEEPALIVE_TAG "keepalive "
 #define MSSFIX_TAG "mssfix"
+#define PING_TAG "ping "
+#define PING_EXIT_TAG "ping-exit "
+#define PING_RESTART_TAG "ping-restart "
 #define PKCS12_TAG "pkcs12 "
 #define PORT_TAG "port "
 #define PROTO_TAG "proto "
@@ -72,6 +97,7 @@
 #define SOCKS_PROXY_TAG "socks-proxy "
 #define SOCKS_PROXY_RETRY_TAG "socks-proxy-retry"
 #define REMOTE_TAG "remote "
+#define REMOTE_RANDOM_TAG "remote-random"
 #define RENEG_SEC_TAG "reneg-sec "
 #define RPORT_TAG "rport "
 #define SECRET_TAG "secret "
@@ -80,6 +106,7 @@
 #define TLS_REMOTE_TAG "tls-remote "
 #define REMOTE_CERT_TLS_TAG "remote-cert-tls "
 #define TUNMTU_TAG "tun-mtu "
+#define ROUTE_TAG "route "
 
 
 static char *
@@ -133,7 +160,7 @@ static gboolean
 handle_path_item (const char *line,
                   const char *tag,
                   const char *key,
-                  NMSettingVPN *s_vpn,
+                  NMSettingVpn *s_vpn,
                   const char *path,
                   char **leftover)
 {
@@ -170,7 +197,7 @@ handle_path_item (const char *line,
 static gboolean
 handle_blob_item (const char ***line,
                   const char *key,
-                  NMSettingVPN *s_vpn,
+                  NMSettingVpn *s_vpn,
                   const char *name,
                   GError **error)
 {
@@ -275,7 +302,7 @@ get_args (const char *line, int *nitems)
 }
 
 static void
-handle_direction (const char *tag, const char *key, char *leftover, NMSettingVPN *s_vpn)
+handle_direction (const char *tag, const char *key, char *leftover, NMSettingVpn *s_vpn)
 {
 	glong direction;
 
@@ -311,6 +338,38 @@ parse_port (const char *str, const char *line)
 	return NULL;
 }
 
+/* returns -1 in case of error */
+static int
+parse_seconds (const char *str, const char *line)
+{
+	glong secs;
+
+	errno = 0;
+	secs = strtol (str, NULL, 10);
+	if ((errno == 0) && (secs >= 0) && (secs <= G_MAXINT))
+		return (int) secs;
+
+	g_warning ("%s: invalid number of seconds in option '%s' - must be in [0, %d]", __func__, line, G_MAXINT);
+	return -1;
+}
+
+static gboolean
+parse_protocol (const char *str, const char *line, gboolean *is_tcp)
+{
+	if (!g_strcmp0 (str, "udp")) {
+		if (is_tcp)
+			*is_tcp = FALSE;
+		return TRUE;
+	} else if (!g_strcmp0 (str, "tcp")) {
+		if (is_tcp)
+			*is_tcp = TRUE;
+		return TRUE;
+	} else {
+		g_warning ("%s: invalid protocol in option '%s'", __func__, line);
+		return FALSE;
+	}
+}
+
 static gboolean
 parse_http_proxy_auth (const char *path,
                        const char *file,
@@ -363,12 +422,59 @@ parse_http_proxy_auth (const char *path,
 	return *out_user && *out_pass;
 }
 
+static gboolean
+handle_num_seconds_item (const char *line,
+                         const char *tag,
+                         const char *key,
+                         NMSettingVpn *s_vpn)
+{
+	char **items = NULL;
+	int nitems;
+
+	if (!strncmp (line, tag, strlen (tag))) {
+		int seconds;
+
+		items = get_args (line + strlen (tag), &nitems);
+		if (nitems == 1) {
+			seconds = parse_seconds (items[0], line);
+			if (seconds >= 0) {
+				char *tmp;
+
+				tmp = g_strdup_printf ("%d", seconds);
+				nm_setting_vpn_add_data_item (s_vpn, key, tmp);
+				g_free (tmp);
+			}
+		} else
+			g_warning ("%s: invalid number of arguments in option '%s', must be one integer", __func__, line);
+
+		g_strfreev (items);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+parse_ip (const char *str, const char *line, guint32 *out_ip)
+{
+	struct in_addr ip;
+
+	if (inet_pton (AF_INET, str, &ip) <= 0) {
+		g_warning ("%s: invalid IP '%s' in option '%s'", __func__, str, line);
+		return FALSE;
+	}
+	if (out_ip)
+		*out_ip = ip.s_addr;
+	return TRUE;
+}
+
 NMConnection *
 do_import (const char *path, char **lines, GError **error)
 {
 	NMConnection *connection = NULL;
 	NMSettingConnection *s_con;
-	NMSettingVPN *s_vpn;
+	NMSettingIPConfig *s_ip4;
+	NMSettingVpn *s_vpn;
 	char *last_dot;
 	char **line;
 	gboolean have_client = FALSE, have_remote = FALSE;
@@ -379,14 +485,17 @@ do_import (const char *path, char **lines, GError **error)
 	gboolean http_proxy = FALSE, socks_proxy = FALSE, proxy_set = FALSE;
 	int nitems;
 
-	connection = nm_connection_new ();
+	connection = nm_simple_connection_new ();
 	s_con = NM_SETTING_CONNECTION (nm_setting_connection_new ());
 	nm_connection_add_setting (connection, NM_SETTING (s_con));
+	s_ip4 = NM_SETTING_IP_CONFIG (nm_setting_ip4_config_new ());
+	nm_connection_add_setting (connection, NM_SETTING (s_ip4));
+	g_object_set (s_ip4, NM_SETTING_IP_CONFIG_METHOD, NM_SETTING_IP4_CONFIG_METHOD_AUTO, NULL);
 
 	s_vpn = NM_SETTING_VPN (nm_setting_vpn_new ());
 
-	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_DBUS_SERVICE_OPENVPN, NULL);
-	
+	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_VPN_SERVICE_TYPE_OPENVPN, NULL);
+
 	/* Get the default path for ca, cert, key file, these files maybe
 	 * in same path with the configuration file */
 	if (g_path_is_absolute (path))
@@ -519,6 +628,11 @@ do_import (const char *path, char **lines, GError **error)
 			continue;
 		}
 
+		if (!strncmp (*line, FLOAT_TAG, strlen (FLOAT_TAG))) {
+			nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_FLOAT, "yes");
+			continue;
+		}
+
 		if (!strncmp (*line, RENEG_SEC_TAG, strlen (RENEG_SEC_TAG))) {
 			items = get_args (*line + strlen (RENEG_SEC_TAG), &nitems);
 
@@ -608,22 +722,37 @@ do_import (const char *path, char **lines, GError **error)
 		if (!strncmp (*line, REMOTE_TAG, strlen (REMOTE_TAG))) {
 			items = get_args (*line + strlen (REMOTE_TAG), &nitems);
 			if (nitems >= 1 && nitems <= 3) {
-				const char *prev = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE);
-				char *new_remote = g_strdup_printf ("%s%s%s", prev ? prev : "", prev ? ", " : "", items[0]);
-				nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE, new_remote);
-				g_free (new_remote);
-				have_remote = TRUE;
+				gboolean ok = TRUE;
+				tmp = NULL;
 
 				if (nitems >= 2) {
 					tmp = parse_port (items[1], *line);
-					if (tmp) {
-						nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_PORT, tmp);
-						g_free (tmp);
+					ok = tmp != NULL;
+					if (ok && nitems == 3)
+						ok = parse_protocol (items[2], *line, NULL);
+				}
+				if (ok) {
+					const char *prev;
+					GString *new_remote = g_string_sized_new (64);
 
-						if (nitems == 3) {
-							 /* TODO */
-						}
+					have_remote = TRUE;
+					prev = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE);
+					if (prev) {
+						g_string_assign (new_remote, prev);
+						g_string_append (new_remote, ", ");
 					}
+					g_string_append (new_remote, items[0]);
+					if (nitems >= 2) {
+						g_string_append_c (new_remote, ':');
+						g_string_append (new_remote, tmp);
+					}
+					if (nitems == 3) {
+						g_string_append_c (new_remote, ':');
+						g_string_append (new_remote, items[2]);
+					}
+					nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE, new_remote->str);
+					g_string_free (new_remote, TRUE);
+					g_free (tmp);
 				}
 			} else
 				g_warning ("%s: invalid number of arguments in option '%s'", __func__, *line);
@@ -632,12 +761,13 @@ do_import (const char *path, char **lines, GError **error)
 			continue;
 		}
 
+		if (!strncmp (*line, REMOTE_RANDOM_TAG, strlen (REMOTE_RANDOM_TAG))) {
+			nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_RANDOM, "yes");
+			continue;
+		}
+
 		if (   !strncmp (*line, PORT_TAG, strlen (PORT_TAG))
 		    || !strncmp (*line, RPORT_TAG, strlen (RPORT_TAG))) {
-			/* Port specified in 'remote' always takes precedence */
-			if (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PORT))
-				continue;
-
 			if (!strncmp (*line, PORT_TAG, strlen (PORT_TAG)))
 				items = get_args (*line + strlen (PORT_TAG), &nitems);
 			else if (!strncmp (*line, RPORT_TAG, strlen (RPORT_TAG)))
@@ -658,6 +788,15 @@ do_import (const char *path, char **lines, GError **error)
 			continue;
 		}
 
+		if (handle_num_seconds_item (*line, PING_TAG, NM_OPENVPN_KEY_PING, s_vpn))
+			continue;
+
+		if (handle_num_seconds_item (*line, PING_EXIT_TAG, NM_OPENVPN_KEY_PING_EXIT, s_vpn))
+			continue;
+
+		if (handle_num_seconds_item (*line, PING_RESTART_TAG, NM_OPENVPN_KEY_PING_RESTART, s_vpn))
+			continue;
+
 		if ( handle_path_item (*line, PKCS12_TAG, NM_OPENVPN_KEY_CA, s_vpn, default_path, NULL) &&
 		     handle_path_item (*line, PKCS12_TAG, NM_OPENVPN_KEY_CERT, s_vpn, default_path, NULL) &&
 		     handle_path_item (*line, PKCS12_TAG, NM_OPENVPN_KEY_KEY, s_vpn, default_path, NULL))
@@ -713,6 +852,33 @@ do_import (const char *path, char **lines, GError **error)
 			continue;
 		}
 
+		if (!strncmp (*line, KEEPALIVE_TAG, strlen (KEEPALIVE_TAG))) {
+			int ping_secs;
+			int ping_restart_secs;
+
+			items = get_args (*line + strlen (KEEPALIVE_TAG), &nitems);
+			if (nitems == 2) {
+				ping_secs = parse_seconds (items[0], *line);
+				ping_restart_secs = parse_seconds (items[1], *line);
+
+				if (ping_secs >= 0 && ping_restart_secs >= 0) {
+					tmp = g_strdup_printf ("%d", ping_secs);
+					tmp2 = g_strdup_printf ("%d", ping_restart_secs);
+
+					nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_PING, tmp);
+					nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_PING_RESTART, tmp2);
+
+					g_free (tmp);
+					g_free (tmp2);
+				} else
+					g_warning ("%s: invalid arguments in option '%s', must be two integers", __func__, *line);
+			} else
+				g_warning ("%s: invalid number of arguments in option '%s', must be two integers", __func__, *line);
+
+			g_strfreev (items);
+			continue;
+		}
+
 		if (!strncmp (*line, KEYSIZE_TAG, strlen (KEYSIZE_TAG))) {
 			items = get_args (*line + strlen (KEYSIZE_TAG), &nitems);
 			if (nitems == 1) {
@@ -784,20 +950,119 @@ do_import (const char *path, char **lines, GError **error)
 			g_strfreev (items);
 			continue;
 		}
+
+#ifdef NM_OPENVPN_OLD
+		if (!strncmp (*line, ROUTE_TAG, strlen (ROUTE_TAG))) {
+			items = get_args (*line + strlen (ROUTE_TAG), &nitems);
+			if (nitems >= 1 && nitems <= 4) {
+				guint32 dest, next_hop, prefix, metric;
+				NMIP4Route *route;
+
+				if (!parse_ip (items[0], *line, &dest))
+					goto route_fail;
+
+				/* init default values */
+				next_hop = 0;
+				prefix = 32;
+				metric = 0;
+				if (nitems >= 2) {
+					if (!parse_ip (items[1], *line, &prefix))
+						goto route_fail;
+					prefix = nm_utils_ip4_netmask_to_prefix (prefix);
+					if (nitems >= 3) {
+						if (!parse_ip (items[2], *line, &next_hop))
+							goto route_fail;
+						if (nitems == 4) {
+							long num;
+							errno = 0;
+							num = strtol (items[3], NULL, 10);
+							if ((errno == 0) && (num >= 0) && (num <= 65535))
+								metric = (guint32) num;
+							else {
+								g_warning ("%s: invalid metric '%s' in option '%s'",
+								           __func__, items[3], *line);
+								goto route_fail;
+							}
+						}
+					}
+				}
+
+				route = nm_ip4_route_new ();
+				nm_ip4_route_set_dest (route, dest);
+				nm_ip4_route_set_prefix (route, prefix);
+				nm_ip4_route_set_next_hop (route, next_hop);
+				nm_ip4_route_set_metric (route, metric);
+				nm_setting_ip4_config_add_route (s_ip4, route);
+				nm_ip4_route_unref (route);
+			} else
+				g_warning ("%s: invalid number of arguments in option '%s'", __func__, *line);
+
+route_fail:
+			g_strfreev (items);
+			continue;
+		}
+#else
+		if (!strncmp (*line, ROUTE_TAG, strlen (ROUTE_TAG))) {
+			items = get_args (*line + strlen (ROUTE_TAG), &nitems);
+			if (nitems >= 1 && nitems <= 4) {
+				guint32 prefix = 32;
+				guint32 metric = 0;
+				const char *dest = items[0];
+				const char *next_hop = "0.0.0.0";
+				NMIPRoute *route;
+
+				if (!parse_ip (items[0], *line, NULL))
+					goto route_fail;
+
+				if (nitems >= 2) {
+					if (!parse_ip (items[1], *line, &prefix))
+						goto route_fail;
+					prefix = nm_utils_ip4_netmask_to_prefix (prefix);
+					if (nitems >= 3) {
+						if (!parse_ip (items[2], *line, NULL))
+							goto route_fail;
+						next_hop = items[2];
+						if (nitems == 4) {
+							long num;
+							errno = 0;
+							num = strtol (items[3], NULL, 10);
+							if ((errno == 0) && (num >= 0) && (num <= 65535))
+								metric = (guint32) num;
+							else {
+								g_warning ("%s: invalid metric '%s' in option '%s'",
+								           __func__, items[3], *line);
+								goto route_fail;
+							}
+						}
+					}
+				}
+
+				route = nm_ip_route_new (AF_INET, dest, prefix, next_hop, metric, NULL);
+				nm_setting_ip_config_add_route (s_ip4, route);
+				nm_ip_route_unref (route);
+			} else
+				g_warning ("%s: invalid number of arguments in option '%s'", __func__, *line);
+
+route_fail:
+			g_strfreev (items);
+			continue;
+		}
+#endif
+
 	}
 
 	if (!have_client && !have_sk) {
-		g_set_error (error,
-		             OPENVPN_PLUGIN_UI_ERROR,
-		             OPENVPN_PLUGIN_UI_ERROR_FILE_NOT_OPENVPN,
-		             "The file to import wasn't a valid OpenVPN client configuration.");
+		g_set_error_literal (error,
+		                     OPENVPN_EDITOR_PLUGIN_ERROR,
+		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     _("The file to import wasn't a valid OpenVPN client configuration."));
 		g_object_unref (connection);
 		connection = NULL;
 	} else if (!have_remote) {
-		g_set_error (error,
-		             OPENVPN_PLUGIN_UI_ERROR,
-		             OPENVPN_PLUGIN_UI_ERROR_FILE_NOT_OPENVPN,
-		             "The file to import wasn't a valid OpenVPN configure (no remote).");
+		g_set_error_literal (error,
+		                     OPENVPN_EDITOR_PLUGIN_ERROR,
+		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     _("The file to import wasn't a valid OpenVPN configure (no remote)."));
 		g_object_unref (connection);
 		connection = NULL;
 	} else {
@@ -865,7 +1130,8 @@ gboolean
 do_export (const char *path, NMConnection *connection, GError **error)
 {
 	NMSettingConnection *s_con;
-	NMSettingVPN *s_vpn;
+	NMSettingIPConfig *s_ip4;
+	NMSettingVpn *s_vpn;
 	FILE *f;
 	const char *value;
 	const char *gateways = NULL;
@@ -878,6 +1144,9 @@ do_export (const char *path, NMConnection *connection, GError **error)
 	const char *static_key = NULL;
 	const char *static_key_direction = NULL;
 	const char *port = NULL;
+	const char *ping = NULL;
+	const char *ping_exit = NULL;
+	const char *ping_restart = NULL;
 	const char *local_ip = NULL;
 	const char *remote_ip = NULL;
 	const char *tls_remote = NULL;
@@ -890,25 +1159,31 @@ do_export (const char *path, NMConnection *connection, GError **error)
 	gboolean success = FALSE;
 	gboolean proto_udp = TRUE;
 	gboolean use_lzo = FALSE;
+	gboolean use_float = FALSE;
 	gboolean reneg_exists = FALSE;
 	guint32 reneg = 0;
 	gboolean keysize_exists = FALSE;
 	guint32 keysize = 0;
+	gboolean randomize_hosts = FALSE;
 	const char *proxy_type = NULL;
 	const char *proxy_server = NULL;
 	const char *proxy_port = NULL;
 	const char *proxy_retry = NULL;
 	const char *proxy_username = NULL;
 	const char *proxy_password = NULL;
+	int i;
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	s_vpn = nm_connection_get_setting_vpn (connection);
 
 	f = fopen (path, "w");
 	if (!f) {
-		g_set_error (error, 0, 0, "could not open file for writing");
+		g_set_error_literal (error,
+		                     OPENVPN_EDITOR_PLUGIN_ERROR,
+		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     _("could not open file for writing"));
 		return FALSE;
 	}
 
@@ -916,7 +1191,10 @@ do_export (const char *path, NMConnection *connection, GError **error)
 	if (value && strlen (value))
 		gateways = value;
 	else {
-		g_set_error (error, 0, 0, "connection was incomplete (missing gateway)");
+		g_set_error_literal (error,
+		                     OPENVPN_EDITOR_PLUGIN_ERROR,
+		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     _("connection was incomplete (missing gateway)"));
 		goto done;
 	}
 
@@ -963,6 +1241,18 @@ do_export (const char *path, NMConnection *connection, GError **error)
 	if (value && strlen (value))
 		port = value;
 
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PING);
+	if (value && strlen (value))
+		ping = value;
+
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PING_EXIT);
+	if (value && strlen (value))
+		ping_exit = value;
+
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PING_RESTART);
+	if (value && strlen (value))
+		ping_restart = value;
+
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS);
 	if (value && strlen (value)) {
 		reneg_exists = TRUE;
@@ -990,6 +1280,10 @@ do_export (const char *path, NMConnection *connection, GError **error)
 	if (value && !strcmp (value, "yes"))
 		use_lzo = TRUE;
 
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FLOAT);
+	if (value && !strcmp (value, "yes"))
+		use_float = TRUE;
+
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CIPHER);
 	if (value && strlen (value))
 		cipher = value;
@@ -1020,21 +1314,44 @@ do_export (const char *path, NMConnection *connection, GError **error)
 	if (value && strlen (value))
 		remote_cert_tls = value;
 
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_RANDOM);
+	if (value && !strcmp (value, "yes"))
+		randomize_hosts = TRUE;
+
 	/* Advanced values end */
 
 	fprintf (f, "client\n");
 
+	/* 'remote' */
 	gw_list = g_strsplit_set (gateways, " ,", 0);
 	for (gw_iter = gw_list; gw_iter && *gw_iter; gw_iter++) {
+		char *tmp_host, *tmp_port,*tmp_proto;
 		if (**gw_iter == '\0')
 			continue;
-		fprintf (f, "remote %s%s%s\n",
+		tmp_host = g_strstrip (*gw_iter);
+		tmp_port = strchr (tmp_host, ':');
+		tmp_proto = tmp_port ? strchr (tmp_port + 1, ':') : NULL;
+		if (tmp_port)
+			*tmp_port++ = '\0';
+		if (tmp_proto)
+			*tmp_proto++ = '\0';
+		if (tmp_port && !*tmp_port)
+			tmp_port = NULL;
+		if (tmp_proto && !*tmp_proto)
+			tmp_proto = NULL;
+
+		fprintf (f, "remote %s%s%s%s%s\n",
 		         *gw_iter,
-		         port ? " " : "",
-		         port ? port : "");
+		         tmp_port ? " " : tmp_proto ? " " : "",
+		         tmp_port ? tmp_port : tmp_proto ? !strcmp (tmp_proto, "udp") ? "1194" : "443": "",
+		         tmp_proto ? " " : "",
+		         tmp_proto ? tmp_proto : "");
 	}
 	g_strfreev (gw_list);
 
+	if (randomize_hosts)
+		fprintf (f, "remote-random\n");
+
 	/* Handle PKCS#12 (all certs are the same file) */
 	if (   cacert && user_cert && private_key
 	    && !strcmp (cacert, user_cert) && !strcmp (cacert, private_key))
@@ -1074,6 +1391,9 @@ do_export (const char *path, NMConnection *connection, GError **error)
 	if (use_lzo)
 		fprintf (f, "comp-lzo yes\n");
 
+	if (use_float)
+		fprintf (f, "float\n");
+
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_MSSFIX);
 	if (value && strlen (value)) {
 		if (!strcmp (value, "yes"))
@@ -1092,6 +1412,17 @@ do_export (const char *path, NMConnection *connection, GError **error)
 	if (device_type)
 		fprintf (f, "dev-type %s\n", device_type);
 	fprintf (f, "proto %s\n", proto_udp ? "udp" : "tcp");
+	if (port)
+		fprintf (f, "port %s\n", port);
+
+	if (ping)
+		fprintf (f, "ping %s\n", ping);
+
+	if (ping_exit)
+		fprintf (f, "ping-exit %s\n", ping_exit);
+
+	if (ping_restart)
+		fprintf (f, "ping-restart %s\n", ping_restart);
 
 	if (local_ip && remote_ip)
 		fprintf (f, "ifconfig %s %s\n", local_ip, remote_ip);
@@ -1160,6 +1491,55 @@ do_export (const char *path, NMConnection *connection, GError **error)
 		}
 	}
 
+#ifdef NM_OPENVPN_OLD
+	/* Route handling is different in libnm-util and libnm */
+	/* Static routes */
+	s_ip4 = nm_connection_get_setting_ip4_config (connection);
+	for (i = 0; s_ip4 && i < nm_setting_ip4_config_get_num_routes (s_ip4); i++) {
+		char dest_str[INET_ADDRSTRLEN];
+		char netmask_str[INET_ADDRSTRLEN];
+		char next_hop_str[INET_ADDRSTRLEN];
+		guint32 dest, netmask, next_hop;
+		NMIP4Route *route = nm_setting_ip4_config_get_route (s_ip4, i);
+
+		dest = nm_ip4_route_get_dest (route);
+		memset (dest_str, '\0', sizeof (dest_str));
+		inet_ntop (AF_INET, (const void *) &dest, dest_str, sizeof (dest_str));
+
+		memset (netmask_str, '\0', sizeof (netmask_str));
+		netmask = nm_utils_ip4_prefix_to_netmask (nm_ip4_route_get_prefix (route));
+		inet_ntop (AF_INET, (const void *) &netmask, netmask_str, sizeof (netmask_str));
+
+		next_hop = nm_ip4_route_get_next_hop (route);
+		memset (next_hop_str, '\0', sizeof (next_hop_str));
+		inet_ntop (AF_INET, (const void *) &next_hop, next_hop_str, sizeof (next_hop_str));
+
+		fprintf (f, "route %s %s %s %u\n",
+		         dest_str,
+		         netmask_str,
+		         next_hop_str,
+		         nm_ip4_route_get_metric (route));
+	}
+#else
+	/* Static routes */
+	s_ip4 = nm_connection_get_setting_ip4_config (connection);
+	for (i = 0; s_ip4 && i < nm_setting_ip_config_get_num_routes (s_ip4); i++) {
+		char netmask_str[INET_ADDRSTRLEN];
+		guint32 netmask;
+		NMIPRoute *route = nm_setting_ip_config_get_route (s_ip4, i);
+
+		memset (netmask_str, '\0', sizeof (netmask_str));
+		netmask = nm_utils_ip4_prefix_to_netmask (nm_ip_route_get_prefix (route));
+		inet_ntop (AF_INET, (const void *) &netmask, netmask_str, sizeof (netmask_str));
+
+		fprintf (f, "route %s %s %s %ld\n",
+		         nm_ip_route_get_dest (route),
+		         netmask_str,
+		         nm_ip_route_get_next_hop (route) ? nm_ip_route_get_next_hop (route) : "0.0.0.0",
+		         nm_ip_route_get_metric (route));
+	}
+#endif
+
 	/* Add hard-coded stuff */
 	fprintf (f,
 	         "nobind\n"
