@@ -28,9 +28,6 @@
 #endif
 
 #include <glib/gi18n.h>
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib-lowlevel.h>
-#include <dbus/dbus-glib.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -51,13 +48,13 @@
 #include <grp.h>
 
 #include <NetworkManager.h>
-#include <NetworkManagerVPN.h>
+#include <nm-vpn-service-plugin.h>
 #include <nm-setting-vpn.h>
 
 #include "nm-openvpn-service.h"
 #include "nm-utils.h"
-#include "common/utils.h"
-#include "common/nm-glib-compat.h"
+#include "../common/utils.h"
+#include "../common/nm-glib-compat.h"
 
 #if !defined(DIST_VERSION)
 # define DIST_VERSION VERSION
@@ -66,9 +63,9 @@
 static gboolean debug = FALSE;
 static GMainLoop *loop = NULL;
 
-#define NM_OPENVPN_HELPER_PATH		LIBEXECDIR"/nm-openvpn-service-openvpn-helper"
+#define NM_OPENVPN_HELPER_PATH LIBEXECDIR"/nm-openvpn-service-openvpn-helper"
 
-G_DEFINE_TYPE (NMOpenvpnPlugin, nm_openvpn_plugin, NM_TYPE_VPN_PLUGIN)
+G_DEFINE_TYPE (NMOpenvpnPlugin, nm_openvpn_plugin, NM_TYPE_VPN_SERVICE_PLUGIN)
 
 #define NM_OPENVPN_PLUGIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_OPENVPN_PLUGIN, NMOpenvpnPluginPrivate))
 
@@ -85,7 +82,7 @@ typedef struct {
 } NMOpenvpnPluginIOData;
 
 typedef struct {
-	GPid	pid;
+	GPid pid;
 	guint connect_timer;
 	guint connect_count;
 	NMOpenvpnPluginIOData *io_data;
@@ -109,10 +106,14 @@ static ValidProperty valid_properties[] = {
 	{ NM_OPENVPN_KEY_KEYSIZE,              G_TYPE_INT, 1, 65535, FALSE },
 	{ NM_OPENVPN_KEY_COMP_LZO,             G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_CONNECTION_TYPE,      G_TYPE_STRING, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_FLOAT,                G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_FRAGMENT_SIZE,        G_TYPE_INT, 0, G_MAXINT, FALSE },
 	{ NM_OPENVPN_KEY_KEY,                  G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_LOCAL_IP,             G_TYPE_STRING, 0, 0, TRUE },
 	{ NM_OPENVPN_KEY_MSSFIX,               G_TYPE_BOOLEAN, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_PING,                 G_TYPE_INT, 0, G_MAXINT, FALSE },
+	{ NM_OPENVPN_KEY_PING_EXIT,            G_TYPE_INT, 0, G_MAXINT, FALSE },
+	{ NM_OPENVPN_KEY_PING_RESTART,         G_TYPE_INT, 0, G_MAXINT, FALSE },
 	{ NM_OPENVPN_KEY_PROTO_TCP,            G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_PORT,                 G_TYPE_INT, 1, 65535, FALSE },
 	{ NM_OPENVPN_KEY_PROXY_TYPE,           G_TYPE_STRING, 0, 0, FALSE },
@@ -250,18 +251,17 @@ validate_one_property (const char *key, const char *value, gpointer user_data)
 }
 
 static gboolean
-nm_openvpn_properties_validate (NMSettingVPN *s_vpn, GError **error)
+nm_openvpn_properties_validate (NMSettingVpn *s_vpn, GError **error)
 {
 	GError *validate_error = NULL;
 	ValidateInfo info = { &valid_properties[0], &validate_error, FALSE };
 
 	nm_setting_vpn_foreach_data_item (s_vpn, validate_one_property, &info);
 	if (!info.have_items) {
-		g_set_error (error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             "%s",
-		             _("No VPN configuration options."));
+		g_set_error_literal (error,
+		                     NM_VPN_PLUGIN_ERROR,
+		                     NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+		                     _("No VPN configuration options."));
 		return FALSE;
 	}
 
@@ -273,7 +273,7 @@ nm_openvpn_properties_validate (NMSettingVPN *s_vpn, GError **error)
 }
 
 static gboolean
-nm_openvpn_secrets_validate (NMSettingVPN *s_vpn, GError **error)
+nm_openvpn_secrets_validate (NMSettingVpn *s_vpn, GError **error)
 {
 	GError *validate_error = NULL;
 	ValidateInfo info = { &valid_secrets[0], &validate_error, FALSE };
@@ -487,10 +487,10 @@ handle_auth (NMOpenvpnPluginIOData *io_data,
 }
 
 static gboolean
-handle_management_socket (NMVPNPlugin *plugin,
+handle_management_socket (NMVpnServicePlugin *plugin,
                           GIOChannel *source,
                           GIOCondition condition,
-                          NMVPNPluginFailure *out_failure)
+                          NMVpnPluginFailure *out_failure)
 {
 	NMOpenvpnPluginPrivate *priv = NM_OPENVPN_PLUGIN_GET_PRIVATE (plugin);
 	gboolean again = TRUE;
@@ -529,7 +529,7 @@ handle_management_socket (NMVPNPlugin *plugin,
 						g_message ("Requesting new secrets: '%s' (%s)", message, joined);
 						g_free (joined);
 					}
-					nm_vpn_plugin_secrets_required (plugin, message, (const char **) hints);
+					nm_vpn_service_plugin_secrets_required (plugin, message, (const char **) hints);
 				} else {
 					/* Interactive not allowed, can't ask for more secrets */
 					g_warning ("More secrets required but cannot ask interactively");
@@ -583,12 +583,11 @@ out:
 static gboolean
 nm_openvpn_socket_data_cb (GIOChannel *source, GIOCondition condition, gpointer user_data)
 {
-	NMVPNPlugin *plugin = NM_VPN_PLUGIN (user_data);
-	NMVPNPluginFailure failure = NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED;
+	NMVpnServicePlugin *plugin = NM_VPN_SERVICE_PLUGIN (user_data);
+	NMVpnPluginFailure failure = NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED;
 
 	if (!handle_management_socket (plugin, source, condition, &failure)) {
-		nm_vpn_plugin_failure (plugin, failure);
-		nm_vpn_plugin_set_state (plugin, NM_VPN_SERVICE_STATE_STOPPED);
+		nm_vpn_service_plugin_failure (plugin, failure);
 		return FALSE;
 	}
 
@@ -610,8 +609,7 @@ nm_openvpn_connect_timer_cb (gpointer data)
 	fd = socket (AF_UNIX, SOCK_STREAM, 0);
 	if (fd < 0) {
 		g_warning ("Could not create management socket");
-		nm_vpn_plugin_failure (NM_VPN_PLUGIN (plugin), NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
-		nm_vpn_plugin_set_state (NM_VPN_PLUGIN (plugin), NM_VPN_SERVICE_STATE_STOPPED);
+		nm_vpn_service_plugin_failure (NM_VPN_SERVICE_PLUGIN (plugin), NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
 		goto out;
 	}
 
@@ -625,8 +623,7 @@ nm_openvpn_connect_timer_cb (gpointer data)
 		priv->connect_timer = 0;
 
 		g_warning ("Could not open management socket");
-		nm_vpn_plugin_failure (NM_VPN_PLUGIN (plugin), NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
-		nm_vpn_plugin_set_state (NM_VPN_PLUGIN (plugin), NM_VPN_SERVICE_STATE_STOPPED);
+		nm_vpn_service_plugin_failure (NM_VPN_SERVICE_PLUGIN (plugin), NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
 	} else {
 		io_data->socket_channel = g_io_channel_unix_new (fd);
 		g_io_channel_set_encoding (io_data->socket_channel, NULL, NULL);
@@ -653,9 +650,9 @@ nm_openvpn_schedule_connect_timer (NMOpenvpnPlugin *plugin)
 static void
 openvpn_watch_cb (GPid pid, gint status, gpointer user_data)
 {
-	NMVPNPlugin *plugin = NM_VPN_PLUGIN (user_data);
+	NMVpnServicePlugin *plugin = NM_VPN_SERVICE_PLUGIN (user_data);
 	NMOpenvpnPluginPrivate *priv = NM_OPENVPN_PLUGIN_GET_PRIVATE (plugin);
-	NMVPNPluginFailure failure = NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED;
+	NMVpnPluginFailure failure = NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED;
 	guint error = 0;
 	gboolean good_exit = FALSE;
 
@@ -676,14 +673,8 @@ openvpn_watch_cb (GPid pid, gint status, gpointer user_data)
 	priv->pid = 0;
 
 	/* OpenVPN doesn't supply useful exit codes :( */
-	switch (error) {
-	case 0:
+	if (error == 0)
 		good_exit = TRUE;
-		break;
-	default:
-		failure = NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED;
-		break;
-	}
 
 	/* Try to get the last bits of data from openvpn */
 	if (priv->io_data && priv->io_data->socket_channel) {
@@ -698,10 +689,10 @@ openvpn_watch_cb (GPid pid, gint status, gpointer user_data)
 		}
 	}
 
-	if (!good_exit)
-		nm_vpn_plugin_failure (plugin, failure);
-
-	nm_vpn_plugin_set_state (plugin, NM_VPN_SERVICE_STATE_STOPPED);
+	if (good_exit)
+		nm_vpn_service_plugin_disconnect (plugin, NULL);
+	else
+		nm_vpn_service_plugin_failure (plugin, failure);
 }
 
 static gboolean
@@ -799,7 +790,7 @@ add_openvpn_arg_int (GPtrArray *args, const char *arg)
 }
 
 static void
-add_cert_args (GPtrArray *args, NMSettingVPN *s_vpn)
+add_cert_args (GPtrArray *args, NMSettingVpn *s_vpn)
 {
 	const char *ca, *cert, *key;
 
@@ -837,7 +828,7 @@ add_cert_args (GPtrArray *args, NMSettingVPN *s_vpn)
 
 static void
 update_io_data_from_vpn_setting (NMOpenvpnPluginIOData *io_data,
-                                 NMSettingVPN *s_vpn,
+                                 NMSettingVpn *s_vpn,
                                  const char *default_username)
 {
 	const char *tmp;
@@ -936,7 +927,7 @@ check_chroot_dir_usability (const char *chdir, const char *user)
 
 static gboolean
 nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
-                                 NMSettingVPN *s_vpn,
+                                 NMSettingVpn *s_vpn,
                                  const char *default_username,
                                  const char *uuid,
                                  GError **error)
@@ -950,37 +941,35 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	char *stmp;
 	const char *defport, *proto_tcp;
 	const char *nm_openvpn_user, *nm_openvpn_group, *nm_openvpn_chroot;
+	gchar *bus_name;
 
 	/* Find openvpn */
 	openvpn_binary = nm_find_openvpn ();
 	if (!openvpn_binary) {
-		g_set_error (error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             "%s",
-		             _("Could not find the openvpn binary."));
+		g_set_error_literal (error,
+		                     NM_VPN_PLUGIN_ERROR,
+		                     NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+		                     _("Could not find the openvpn binary."));
 		return FALSE;
 	}
-  
- 	auth = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_AUTH);
- 	if (auth) {
- 		if (!validate_auth(auth)) {
- 			g_set_error (error,
- 			             NM_VPN_PLUGIN_ERROR,
- 			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
- 			             "%s",
- 			             _("Invalid HMAC auth."));
- 			return FALSE;
- 		}
- 	}
+
+	auth = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_AUTH);
+	if (auth) {
+		if (!validate_auth(auth)) {
+			g_set_error_literal (error,
+			                     NM_VPN_PLUGIN_ERROR,
+			                     NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			                     _("Invalid HMAC auth."));
+			return FALSE;
+		}
+	}
 
 	connection_type = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
 	if (!validate_connection_type (connection_type)) {
-		g_set_error (error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             "%s",
-		             _("Invalid connection type."));
+		g_set_error_literal (error,
+		                     NM_VPN_PLUGIN_ERROR,
+		                     NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+		                     _("Invalid connection type."));
 		return FALSE;
 	}
 
@@ -1034,8 +1023,10 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 					add_openvpn_arg (args, "1194"); /* default IANA port */
 
 				if (proto) {
-					if (!strcmp (proto, "udp") || !strcmp (proto, "tcp"))
-						add_openvpn_arg (args, proto);
+					if (!strcmp (proto, "udp"))
+							add_openvpn_arg (args, proto);
+					else if (!strcmp (proto, "tcp"))
+							add_openvpn_arg (args, "tcp-client");
 					else {
 						g_set_error (error,
 						             NM_VPN_PLUGIN_ERROR,
@@ -1046,7 +1037,7 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 						return FALSE;
 					}
 				} else if (proto_tcp && !strcmp (proto_tcp, "yes"))
-					add_openvpn_arg (args, "tcp");
+					add_openvpn_arg (args, "tcp-client");
 				else
 					add_openvpn_arg (args, "udp");
 			}
@@ -1092,6 +1083,53 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	if (tmp && !strcmp (tmp, "yes"))
 		add_openvpn_arg (args, "--comp-lzo");
 
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FLOAT);
+	if (tmp && !strcmp (tmp, "yes"))
+		add_openvpn_arg (args, "--float");
+
+	/* ping, ping-exit, ping-restart */
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PING);
+	if (tmp) {
+		add_openvpn_arg (args, "--ping");
+		if (!add_openvpn_arg_int (args, tmp)) {
+			g_set_error (error,
+			             NM_VPN_PLUGIN_ERROR,
+			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			             _("Invalid ping duration '%s'."),
+			             tmp);
+			free_openvpn_args (args);
+			return FALSE;
+		}
+	}
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PING_EXIT);
+	if (tmp) {
+		add_openvpn_arg (args, "--ping-exit");
+		if (!add_openvpn_arg_int (args, tmp)) {
+			g_set_error (error,
+			             NM_VPN_PLUGIN_ERROR,
+			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			             _("Invalid ping-exit duration '%s'."),
+			             tmp);
+			free_openvpn_args (args);
+			return FALSE;
+		}
+	}
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PING_RESTART);
+	if (tmp) {
+		add_openvpn_arg (args, "--ping-restart");
+		if (!add_openvpn_arg_int (args, tmp)) {
+			g_set_error (error,
+			             NM_VPN_PLUGIN_ERROR,
+			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			             _("Invalid ping-restart duration '%s'."),
+			             tmp);
+			free_openvpn_args (args);
+			return FALSE;
+		}
+	}
+
 	add_openvpn_arg (args, "--nobind");
 
 	/* Device and device type, defaults to tun */
@@ -1254,8 +1292,9 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 
 	/* Up script, called when connection has been established or has been restarted */
 	add_openvpn_arg (args, "--up");
-	stmp = g_strdup_printf ("%s%s %s --", NM_OPENVPN_HELPER_PATH, debug ? " --helper-debug" : "",
-	                        dev_type_is_tap ? "--tap" : "--tun");
+	g_object_get (plugin, NM_VPN_SERVICE_PLUGIN_DBUS_SERVICE_NAME, &bus_name, NULL);
+	stmp = g_strdup_printf ("%s%s --bus-name %s %s --", NM_OPENVPN_HELPER_PATH, debug ? " --helper-debug" : "",
+	                        bus_name, dev_type_is_tap ? "--tap" : "--tun");
 	add_openvpn_arg (args, stmp);
 	g_free (stmp);
 	add_openvpn_arg (args, "--up-restart");
@@ -1267,7 +1306,6 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	/* Management socket for localhost access to supply username and password */
 	add_openvpn_arg (args, "--management");
 	g_warn_if_fail (priv->mgt_path == NULL);
-	g_free (priv->mgt_path);
 	priv->mgt_path = g_strdup_printf (LOCALSTATEDIR "/run/NetworkManager/nm-openvpn-%s", uuid);
 	add_openvpn_arg (args, priv->mgt_path);
 	add_openvpn_arg (args, "unix");
@@ -1305,11 +1343,10 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP);
 		if (!tmp) {
 			/* Insufficient data (FIXME: this should really be detected when validating the properties */
-			g_set_error (error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             "%s",
-			             _("Missing required local IP address for static key mode."));
+			g_set_error_literal (error,
+			                     NM_VPN_PLUGIN_ERROR,
+			                     NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			                     _("Missing required local IP address for static key mode."));
 			free_openvpn_args (args);
 			return FALSE;
 		}
@@ -1318,11 +1355,10 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP);
 		if (!tmp) {
 			/* Insufficient data (FIXME: this should really be detected when validating the properties */
-			g_set_error (error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             "%s",
-			             _("Missing required remote IP address for static key mode."));
+			g_set_error_literal (error,
+			                     NM_VPN_PLUGIN_ERROR,
+			                     NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			                     _("Missing required remote IP address for static key mode."));
 			free_openvpn_args (args);
 			return FALSE;
 		}
@@ -1444,7 +1480,7 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 }
 
 static const char *
-check_need_secrets (NMSettingVPN *s_vpn, gboolean *need_secrets)
+check_need_secrets (NMSettingVpn *s_vpn, gboolean *need_secrets)
 {
 	const char *tmp, *key, *ctype;
 	NMSettingSecretFlags secret_flags = NM_SETTING_SECRET_FLAG_NONE;
@@ -1499,20 +1535,58 @@ check_need_secrets (NMSettingVPN *s_vpn, gboolean *need_secrets)
 }
 
 static gboolean
-_connect_common (NMVPNPlugin   *plugin,
+ensure_killed (gpointer data)
+{
+	int pid = GPOINTER_TO_INT (data);
+
+	if (kill (pid, 0) == 0)
+		kill (pid, SIGKILL);
+
+	return FALSE;
+}
+
+static gboolean
+real_disconnect (NMVpnServicePlugin	 *plugin,
+			  GError		**err)
+{
+	NMOpenvpnPluginPrivate *priv = NM_OPENVPN_PLUGIN_GET_PRIVATE (plugin);
+
+	if (priv->pid) {
+		if (kill (priv->pid, SIGTERM) == 0)
+			g_timeout_add (2000, ensure_killed, GINT_TO_POINTER (priv->pid));
+		else
+			kill (priv->pid, SIGKILL);
+
+		g_message ("Terminated openvpn daemon with PID %d.", priv->pid);
+		priv->pid = 0;
+	}
+
+	g_free (priv->mgt_path);
+	priv->mgt_path = NULL;
+
+	return TRUE;
+}
+
+static gboolean
+_connect_common (NMVpnServicePlugin   *plugin,
                  NMConnection  *connection,
-                 GHashTable    *details,
+                 GVariant      *details,
                  GError       **error)
 {
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	const char *connection_type;
 	const char *user_name;
 
+	if (!real_disconnect (plugin, error)) {
+		g_warning ("Could not clean up previous daemon run: %s", (*error)->message);
+		g_clear_error (error);
+	}
+
 	s_vpn = nm_connection_get_setting_vpn (connection);
 	if (!s_vpn) {
 		g_set_error_literal (error,
 		                     NM_VPN_PLUGIN_ERROR,
-		                     NM_VPN_PLUGIN_ERROR_CONNECTION_INVALID,
+		                     NM_VPN_PLUGIN_ERROR_INVALID_CONNECTION,
 		                     _("Could not process the request because the VPN connection settings were invalid."));
 		return FALSE;
 	}
@@ -1521,7 +1595,7 @@ _connect_common (NMVPNPlugin   *plugin,
 	if (!validate_connection_type (connection_type)) {
 		g_set_error_literal (error,
 		                     NM_VPN_PLUGIN_ERROR,
-		                     NM_VPN_PLUGIN_ERROR_CONNECTION_INVALID,
+		                     NM_VPN_PLUGIN_ERROR_INVALID_CONNECTION,
 		                     _("Could not process the request because the openvpn connection type was invalid."));
 		return FALSE;
 	}
@@ -1547,7 +1621,7 @@ _connect_common (NMVPNPlugin   *plugin,
 }
 
 static gboolean
-real_connect (NMVPNPlugin   *plugin,
+real_connect (NMVpnServicePlugin   *plugin,
               NMConnection  *connection,
               GError       **error)
 {
@@ -1555,9 +1629,9 @@ real_connect (NMVPNPlugin   *plugin,
 }
 
 static gboolean
-real_connect_interactive (NMVPNPlugin   *plugin,
+real_connect_interactive (NMVpnServicePlugin   *plugin,
                           NMConnection  *connection,
-                          GHashTable    *details,
+                          GVariant      *details,
                           GError       **error)
 {
 	if (!_connect_common (plugin, connection, details, error))
@@ -1568,16 +1642,16 @@ real_connect_interactive (NMVPNPlugin   *plugin,
 }
 
 static gboolean
-real_need_secrets (NMVPNPlugin *plugin,
+real_need_secrets (NMVpnServicePlugin *plugin,
                    NMConnection *connection,
-                   char **setting_name,
+                   const char **setting_name,
                    GError **error)
 {
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	const char *connection_type;
 	gboolean need_secrets = FALSE;
 
-	g_return_val_if_fail (NM_IS_VPN_PLUGIN (plugin), FALSE);
+	g_return_val_if_fail (NM_IS_VPN_SERVICE_PLUGIN (plugin), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
 
 	if (debug) {
@@ -1589,7 +1663,7 @@ real_need_secrets (NMVPNPlugin *plugin,
 	if (!s_vpn) {
 		g_set_error_literal (error,
 		                     NM_VPN_PLUGIN_ERROR,
-		                     NM_VPN_PLUGIN_ERROR_CONNECTION_INVALID,
+		                     NM_VPN_PLUGIN_ERROR_INVALID_CONNECTION,
 		                     _("Could not process the request because the VPN connection settings were invalid."));
 		return FALSE;
 	}
@@ -1610,12 +1684,12 @@ real_need_secrets (NMVPNPlugin *plugin,
 }
 
 static gboolean
-real_new_secrets (NMVPNPlugin *plugin,
+real_new_secrets (NMVpnServicePlugin *plugin,
                   NMConnection *connection,
                   GError **error)
 {
 	NMOpenvpnPluginPrivate *priv = NM_OPENVPN_PLUGIN_GET_PRIVATE (plugin);
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	const char *message = NULL;
 	char **hints = NULL;
 
@@ -1623,7 +1697,7 @@ real_new_secrets (NMVPNPlugin *plugin,
 	if (!s_vpn) {
 		g_set_error_literal (error,
 		                     NM_VPN_PLUGIN_ERROR,
-		                     NM_VPN_PLUGIN_ERROR_CONNECTION_INVALID,
+		                     NM_VPN_PLUGIN_ERROR_INVALID_CONNECTION,
 		                     _("Could not process the request because the VPN connection settings were invalid."));
 		return FALSE;
 	}
@@ -1637,7 +1711,7 @@ real_new_secrets (NMVPNPlugin *plugin,
 	if (!handle_auth (priv->io_data, priv->io_data->pending_auth, &message, &hints)) {
 		g_set_error_literal (error,
 		                     NM_VPN_PLUGIN_ERROR,
-		                     NM_VPN_PLUGIN_ERROR_GENERAL,
+		                     NM_VPN_PLUGIN_ERROR_FAILED,
 		                     _("Unhandled pending authentication."));
 		return FALSE;
 	}
@@ -1646,43 +1720,13 @@ real_new_secrets (NMVPNPlugin *plugin,
 	if (message) {
 		if (debug)
 			g_message ("Requesting new secrets: '%s'", message);
-		nm_vpn_plugin_secrets_required (plugin, message, (const char **) hints);
+		nm_vpn_service_plugin_secrets_required (plugin, message, (const char **) hints);
 	}
 	if (hints)
 		g_free (hints);  /* elements are 'const' */
 	return TRUE;
 }
 
-static gboolean
-ensure_killed (gpointer data)
-{
-	int pid = GPOINTER_TO_INT (data);
-
-	if (kill (pid, 0) == 0)
-		kill (pid, SIGKILL);
-
-	return FALSE;
-}
-
-static gboolean
-real_disconnect (NMVPNPlugin	 *plugin,
-			  GError		**err)
-{
-	NMOpenvpnPluginPrivate *priv = NM_OPENVPN_PLUGIN_GET_PRIVATE (plugin);
-
-	if (priv->pid) {
-		if (kill (priv->pid, SIGTERM) == 0)
-			g_timeout_add (2000, ensure_killed, GINT_TO_POINTER (priv->pid));
-		else
-			kill (priv->pid, SIGKILL);
-
-		g_message ("Terminated openvpn daemon with PID %d.", priv->pid);
-		priv->pid = 0;
-	}
-
-	return TRUE;
-}
-
 static void
 nm_openvpn_plugin_init (NMOpenvpnPlugin *plugin)
 {
@@ -1692,7 +1736,7 @@ static void
 nm_openvpn_plugin_class_init (NMOpenvpnPluginClass *plugin_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (plugin_class);
-	NMVPNPluginClass *parent_class = NM_VPN_PLUGIN_CLASS (plugin_class);
+	NMVpnServicePluginClass *parent_class = NM_VPN_SERVICE_PLUGIN_CLASS (plugin_class);
 
 	g_type_class_add_private (object_class, sizeof (NMOpenvpnPluginPrivate));
 
@@ -1706,7 +1750,7 @@ nm_openvpn_plugin_class_init (NMOpenvpnPluginClass *plugin_class)
 
 static void
 plugin_state_changed (NMOpenvpnPlugin *plugin,
-                      NMVPNServiceState state,
+                      NMVpnServiceState state,
                       gpointer user_data)
 {
 	NMOpenvpnPluginPrivate *priv = NM_OPENVPN_PLUGIN_GET_PRIVATE (plugin);
@@ -1730,16 +1774,21 @@ plugin_state_changed (NMOpenvpnPlugin *plugin,
 }
 
 NMOpenvpnPlugin *
-nm_openvpn_plugin_new (void)
+nm_openvpn_plugin_new (const char *bus_name)
 {
 	NMOpenvpnPlugin *plugin;
+	GError *error = NULL;
 
-	plugin =  (NMOpenvpnPlugin *) g_object_new (NM_TYPE_OPENVPN_PLUGIN,
-	                                            NM_VPN_PLUGIN_DBUS_SERVICE_NAME,
-	                                            NM_DBUS_SERVICE_OPENVPN,
-	                                            NULL);
-	if (plugin)
+	plugin =  (NMOpenvpnPlugin *) g_initable_new (NM_TYPE_OPENVPN_PLUGIN, NULL, &error,
+	                                              NM_VPN_SERVICE_PLUGIN_DBUS_SERVICE_NAME, bus_name,
+	                                              NULL);
+
+	if (plugin) {
 		g_signal_connect (G_OBJECT (plugin), "state-changed", G_CALLBACK (plugin_state_changed), NULL);
+	} else {
+		g_warning ("Failed to initialize a plugin instance: %s", error->message);
+		g_error_free (error);
+	}
 
 	return plugin;
 }
@@ -1766,7 +1815,7 @@ setup_signals (void)
 }
 
 static void
-quit_mainloop (NMVPNPlugin *plugin, gpointer user_data)
+quit_mainloop (NMVpnServicePlugin *plugin, gpointer user_data)
 {
 	g_main_loop_quit ((GMainLoop *) user_data);
 }
@@ -1777,10 +1826,13 @@ main (int argc, char *argv[])
 	NMOpenvpnPlugin *plugin;
 	gboolean persist = FALSE;
 	GOptionContext *opt_ctx = NULL;
+	gchar *bus_name = NM_DBUS_SERVICE_OPENVPN;
+	GError *error = NULL;
 
 	GOptionEntry options[] = {
 		{ "persist", 0, 0, G_OPTION_ARG_NONE, &persist, N_("Don't quit when VPN connection terminates"), NULL },
 		{ "debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable verbose debug logging (may expose passwords)"), NULL },
+		{ "bus-name", 0, 0, G_OPTION_ARG_STRING, &bus_name, N_("D-Bus name to use for this instance"), NULL },
 		{NULL}
 	};
 
@@ -1803,9 +1855,15 @@ main (int argc, char *argv[])
 	g_option_context_add_main_entries (opt_ctx, options, NULL);
 
 	g_option_context_set_summary (opt_ctx,
-		_("nm-openvpn-service provides integrated OpenVPN capability to NetworkManager."));
+	                              _("nm-openvpn-service provides integrated "
+	                                "OpenVPN capability to NetworkManager."));
 
-	g_option_context_parse (opt_ctx, &argc, &argv, NULL);
+	if (!g_option_context_parse (opt_ctx, &argc, &argv, &error)) {
+		g_warning ("Error parsing the command line options: %s", error->message);
+		g_option_context_free (opt_ctx);
+		g_clear_error (&error);
+		exit (1);
+	}
 	g_option_context_free (opt_ctx);
 
 	if (getenv ("OPENVPN_DEBUG"))
@@ -1818,7 +1876,7 @@ main (int argc, char *argv[])
 	    && (system ("/sbin/modprobe tun") == -1))
 		exit (EXIT_FAILURE);
 
-	plugin = nm_openvpn_plugin_new ();
+	plugin = nm_openvpn_plugin_new (bus_name);
 	if (!plugin)
 		exit (EXIT_FAILURE);
 
